"ObjectDomain Application Module"

"""
 Copyright (c) 2001 by Object Domain Systems Inc.
 All rights reserved.

 This script is provided as part of ObjectDomain. Derivations and
 extensions to the source code are permitted provided (a) the derivative
 works are used in conjunction with ObjectDomain and (b) this copyright
 notice is retained in its entirety. Any use of this script, in part or full,
 for the benefit of competitive products is strictly prohibited.
"""

import string
from java.lang import Class
from objectdomain.domain import Domain
from objectdomain.domain import DomainViewHandler
from objectdomain.utility import OdsUtil
from objectdomain.uml.utility import UmlUtil
from objectdomain.domain import BeanEditorManager
from objectdomain.domain.commands import NewModelCmd
from objectdomain.domain.commands import SaveCmd
from objectdomain.domain.commands import SaveAsCmd
from objectdomain.domain.commands import SaveAsSingleFileCmd
from objectdomain.domain.commands import HelpCmd
from objectdomain.domain.commands import SelectInBrowserCmd
from objectdomain.domain.commands import ShowAboutCmd
from objectdomain.domain.commands import SaveAppConfigCmd
from objectdomain.domain.commands import OpenDiagramCmd
from objectdomain.domain.commands import OpenModelCmd
from objectdomain.domain.commands import CloseModelCmd
from objectdomain.uml.command import DeleteCmd
from objectdomain.domain.scripting import ScriptFacility
from objectdomain.uml.mechanisms import UmlWorld
from objectdomain.uml.command import ModifyPropertyCmd
from objectdomain.domain.diagram.command import RefreshDiagramsCmd
from objectdomain.versioncontrol import IBaseVersionControl
from objectdomain.versioncontrol import VersionControlFacility
from objectdomain.domain import Domain
from objectdomain.domain.commands import ExportDiagramCmd, CreateClassDiagramCmd
from objectdomain.uml import command
from objectdomain.uml.core import IElement, IElementCollection
from java.lang import Object
import objectdomain

class OdApp:
  "ObjectDomain Application Class"

  ELEMENT_REF_CHAR = "."

  def __init__(self, domainApp=None):
    if domainApp != None:
      self.domain = domainApp
    else:
      self.domain = Domain.instance()
    self.consoleVersionControlHandler = None

  def get(self, uidOrTypeQualifiedName=None):
    "Get element by UID or qualifed name with optional type (using '@') qualification."
    if uidOrTypeQualifiedName != None:
      if isinstance(uidOrTypeQualifiedName, type('')):
        return nameToElement(uidOrTypeQualifiedName)
      else:
        pass  # todo - handle UIDs
    else:
      return self.getFirstSelectedElement()

  def set(self, target, propertyName, newValue):
    "Set value of a bean property."
    if isinstance(target, Object):
      bean = target
    else:
      bean = self.get(target);
    if (bean == None):
      print "Unable to determine target for command"
      self.beep()
      return

    s = self.domain.getModelSelection()
    cmd = ModifyPropertyCmd(None, s, bean, propertyName, newValue)
    self.domain.addAndDo(cmd)

  def delete(self, o=None):
    if o != None:
      self.select(o)
    s = self.domain.getModelSelection()
    cmd = DeleteCmd(None, s)
    self.domain.addAndDo(cmd)

  def edit(self, target=None):
    if isinstance(target, objectdomain.uml.core.Element):
      element = target
    else:
      element = self.get(target);
    if (element == None):
      print "Unable to determine target for command"
      self.beep()
      return
    bm = BeanEditorManager.instance()
    cmd = bm.getEditorInvocationCmd(element)
    self.domain.addAndDo(cmd)

  def openDiagram(self, target):
    if isinstance(target, objectdomain.uml.core.Element):
      element = target
    else:
      element = self.get(target);
    if (element == None):
      print "Unable to determine target for command"
      self.beep()
      return
    cmd = OpenDiagramCmd(element, self.domain.getCommandProcessor())
    self.domain.addAndDo(cmd)

  def beep(self):
    OdsUtil.beep()

  def setConsoleVersionControlHandler(self, consoleVCHandler) :
    self.consoleVersionControlHandler = consoleVCHandler

  def setVersionControlMgr(self, baseImpl):
    VersionControlFacility.instance().setBaseImpl(baseImpl)

  def vc(self, cmdStr, includeFiles=1):
    if self.consoleVersionControlHandler != None:
      self.consoleVersionControlHandler.execute(cmdStr, includeFiles)
    else:
      print """Error: Console version control handler not configured. See odUserSetup.py"""

  def getAppFrame(self):
    return self.domain.getComponentFrame()

  def openModel(self, filename=None):
    cmd = OpenModelCmd(filename)
    self.domain.addAndDo(cmd)

  def closeModel(self):
    cmd = CloseModelCmd()
    self.domain.addAndDo(cmd)

  def newModel(self):
    cmd = NewModelCmd()
    self.domain.addAndDo(cmd)

  def refreshDiagrams(self):
    cmd = RefreshDiagramsCmd(self.domain)
    self.domain.addAndDo(cmd)

  def save(self):
    cmd = SaveCmd()
    self.domain.addAndDo(cmd)

  def saveAs(self):
    cmd = SaveAsCmd()
    self.domain.addAndDo(cmd)

  def saveAsSingleFile(self):
    cmd = SaveAsSingleFileCmd()
    self.domain.addAndDo(cmd)

  def getCurrentModel(self):
    "get current UML model"
    return self.domain.getModel().getUmlModel()

  def refreshBrowsers(self):
    DomainViewHandler.instance().refreshBrowsers()

  def showHelp(self):
    cmd = HelpCmd()
    self.domain.addAndDo(cmd)

  def showAbout(self):
    cmd = ShowAboutCmd()
    self.domain.addAndDo(cmd)

  def saveAppConfig(self):
    cmd = SaveAppConfigCmd()
    self.domain.addAndDo(cmd)

  # todo: handle selecting collections
  def select(self, o):
    e = self.getElementOrCollectionFrom(o)
    SelectInBrowserCmd(e).handleDo()

  def undo(self):
    self.domain.getGUIHandler().executeUndoCmd()

  def redo(self):
    self.domain.getGUIHandler().executeRedoCmd()

  def copy(self):
    self.domain.getGUIHandler().executeCopyCmd()

  def paste(self):
    self.domain.getGUIHandler().executePasteCmd()

  def refreshDiagrams(self):
    cmd = RefreshDiagramsCmd(self.domain)
    self.domain.addAndDo(cmd)

  s ="""
  def moveView(self, uidOrTypeQualifiedName=None, x, y):
    self.select(uidOrTypeQualifiedName)
    s = self.domain.getModelSelection()
    cmd = MoveCmd(None, s, x, y)
    self.domain.addAndDo(cmd)  """

  def createElement(self, elementTypeName, name=None, owner=None):
    cmd = None
    cmdStr = 'cmd = domain.commands.Create' + elementTypeName + 'Cmd()'
    try:
      exec(cmdStr)
    except:
      cmdStr = 'cmd = command.Create' + elementTypeName + 'Cmd()'
      try:
        exec(cmdStr)
      except:
        try:
          cmdStr = 'cmd = command.CreateUml' + elementTypeName + 'Cmd()'
          exec(cmdStr)
        except:
         print 'Creation error - unknown element type: ' + elementTypeName
         return None
    self.doCmd(cmd, name, owner)
    return cmd.getCreatedElement()

  def createClassDiagram(self):
    cmd = CreateClassDiagramCmd()
    Domain.instance().addAndDo(cmd)

  def export(self, filename=None):
    cmd = ExportDiagramCmd(filename)
    Domain.instance().addAndDo(cmd)

  # *** helper methods ***

  def getCreatedElementOwner(self, owner):
    firstSelected = self.getFirstSelectedElement()
    if owner == None:
      return firstSelected
    else:
      owner = self.getElementOrCollectionFrom(owner)
      if isinstance(owner, IElementCollection):
        firstSelected
      else:
        return owner

  def doCmd(self, cmd, name, owner):
    cmd.initialize(None, self.getSelection())
    owner = self.getCreatedElementOwner(owner)
    cmd.setPotentialOwner(owner)
    if name != None:
      cmd.setObjectName(name)
    if cmd.handleCanDo():
      Domain.instance().addAndDo(cmd)
    else:
      print "Unable to execute command: handleCanDo returned false"
      OdsUtil.beep()

  def getElementOrCollectionFrom(self, o):
    if isinstance(o, IElement):
      return o
    elif isinstance(o, IElementCollection):
      return o
    else:
      return nameToElement(o)

  def isDotQualifiedName(self, o):
    if isinstance(o, type('')):
      return string.find(o, OdApp.ELEMENT_REF_CHAR) != 0
    else:
      return 0

  def getSelection(self):
    return self.domain.getModelSelection()

  def getFirstSelectedElement(self):
    return self.domain.getModelSelection().getElement()

  def getSelectedElements(self):
    return self.domain.getModelSelection().getElements()


# todo move this to java code
# todo throw Exception
def nameToElement(qualifiedName):
  ELEMENT_REF_CHAR = "."

  if string.find(qualifiedName, ELEMENT_REF_CHAR) == 0:
    qualifiedName = qualifiedName[1:]
  if qualifiedName == "":
    return UmlWorld.instance()

  typename = "objectdomain.uml.core.Element"
  #typename = getFullTypeName(type)  - todo
  clazz = Class.forName(typename)

  if qualifiedName == "Model" or qualifiedName[:6] == "Model.":
    atom = UmlWorld.instance().retrieveByNameAndType(qualifiedName, clazz)
  else:
    logicalView = Domain.instance().getModel().getUmlModel().getLogicalView();
    atom = logicalView.retrieveByNameAndType(qualifiedName, clazz)

  if qualifiedName != None and atom == None:
    print "unable to determine target from qualified name: " + qualifiedName

  return atom




