# (c) 2000 phil@bluedynamics.com 
#  utility methods to query ZClass internals
#  v1.1.0
#  v1.2.0 support for nested classes and multiple products
#  tested with Zope 2.2.2 and 2.2.4 , 2.3.[01]
#  uses Zope and ObjectDomain internals so changes can be necessary for future Zope releases

import urllib
import string
import java.lang.Exception
import odUtil
import objectdomain.uml.core.UmlAggregation

def createClasses(od,classes,processSource=0):
   package=od.get()
   if not string.find(package.__class__.__name__,'UmlPackage'):
      raise Exception('Please select a package and retry')
   
   for n in classes.keys():
      od.select(package)
      cl=od.createElement('Class',owner=None,name=n)
      classes[n]['UmlClass']=cl
      for mt in classes[n]['methods']:
         #print n,mt
         od.select(classes[n]['UmlClass'])
         meth=od.createElement('Operation',owner=None,name=mt['name'])
 
         if mt.has_key('params') and mt['params']!= None:
            #print 'class:',n,'params:',mt['params']
            par=mt['params']

            if type(par)==type(''):
               od.select(meth)
               od.createElement('Parameter',owner=None,name=par)
            else:
               for p in mt['params']:
                  od.select(meth)
                  od.createElement('Parameter',owner=None,name=p)
         if mt.has_key('doc') and mt['doc']!=None:
            meth.setDocumentation(mt['doc'])
         if mt.has_key('body') and mt['body']!=None and processSource:
            meth.setSemantics(mt['body'])

def createAggregations(od,classes):
   for n in classes.keys():
      cl=classes[n]['UmlClass']
      for b in classes[n]['has_a']:

         try:
            cl1=classes[b]['UmlClass']
         except:
            continue
         od.select(cl)
         ass=od.createElement('Association')
         it=ass.startConnections()
         assend1=it.nextElement()
         assend2=it.nextElement()
         cl.addAssocEnd(assend1)
         cl1.addAssocEnd(assend2)
         assend1.setAggregation(objectdomain.uml.core.UmlAggregation.AGGREGATE)
         #assend2.setOwner(cl1)
         
def createComposites(od,classes):
   for n in classes.keys():
      cl=classes[n]['UmlClass']
      for b in classes[n]['nested']:

         try:
            cl1=classes[b]['UmlClass']
         except:
            continue
         od.select(cl)
         ass=od.createElement('Association')
         it=ass.startConnections()
         assend1=it.nextElement()
         assend2=it.nextElement()
         cl.addAssocEnd(assend1)
         cl1.addAssocEnd(assend2)
         assend1.setAggregation(objectdomain.uml.core.UmlAggregation.COMPOSITE)
         #assend2.setOwner(cl1)
         

def createGeneralizations(od,classes):
   for n in classes.keys():
      cl=classes[n]['UmlClass']
      for b in classes[n]['bases']:
         #print 'bases:',b
         try:
            bcl=classes[b]['UmlClass']
         except:
            continue
         od.select(cl)
         gen=od.createElement('Generalization')
         gen.setSubtype(cl)
         gen.setSupertype(bcl)
         
      

def reverseZClass(od,host,port,product,processSource=0,processAggregations=1):
   #u=urllib.urlopen('http://localhost:10080/zclassInfo/getClassInfos?product=Mondeal')
   u=urllib.urlopen('http://'+host+':'+str(port)+'/zclassInfo/getClassInfos?product='+product)
   res=eval(u.read())
   createClasses(od,res,processSource)
   createGeneralizations(od,res)
   if processAggregations:
      createAggregations(od,res)
      createComposites(od,res)


def getZClassString(od,host,port,product):
   #u=urllib.urlopen('http://localhost:10080/zclassInfo/getClassInfos?product=Mondeal')
   u=urllib.urlopen('http://'+host+':'+str(port)+'/zclassInfo/getClassInfos?product='+product)
   return u.read()

from java.awt import Dialog,Checkbox,Frame,Button,Label,TextField, FlowLayout,GridLayout,Panel,BorderLayout

class ReverseForm(Frame):
    def __init__(self,od=None):
        self.od=od
        return self.init()
                     
    
    def init(self):
        self.ok=0
        #self.setModal(1)
        self.setBounds(100,100,500,250)
        self.setTitle('ZClass Reverse engineering tool (C) phil@bluedynamics.com')
        self.setLayout(BorderLayout())
        p=Panel()
        p.setLayout(GridLayout(6,2))
        self.add(p,'Center')
        
        p.add(Label('Host:'))
        self.host=TextField('localhost')        
        p.add(self.host)

        p.add(Label('Port:'))
        self.port=TextField('80')
        p.add(self.port)
        
        p.add(Label('Product name:'))
        self.product=TextField('')
        p.add(self.product)

        p.add(Label('insert source code:'))
        self.sourceToo=Checkbox('source')
        p.add(self.sourceToo)
                      
        p.add(Label('reverse engineer aggregations:'))
        self.processAggregations=Checkbox('aggregations')
        p.add(self.processAggregations)
        self.processAggregations.setState(1)

        p.add(Button('Start',actionPerformed=self.startIt))
        p.add(Button('Cancel',actionPerformed=self.cancelIt))

        self.label=Label('click start to begin with reverse engineering')
        self.add('South',self.label)        

        try:
           self.host.setText(defaults['host'])
           self.port.setText(defaults['port'])
           self.product.setText(defaults['product'])
           self.sourceToo.setState(defaults['sourceToo'])
           self.processAggregations.setState(defaults['processAggregations'])
        except:
           pass
        
    def startIt(self,src):
        global defaults
        defaults={}
        defaults['host']=self.host.getText()
        defaults['port']=self.port.getText()
        defaults['product']=self.product.getText()
        defaults['sourceToo']=self.sourceToo.getState()
        defaults['processAggregations']=self.processAggregations.getState()
        
        try:
           self.label.setText('reverse engineering in progress ('+self.product.getText()+'), please wait...')
           reverseZClass(self.od,self.host.getText(),self.port.getText(),
                         self.product.getText(),processSource=self.sourceToo.getState(),processAggregations=self.processAggregations.getState())
           self.label.setText('ready.')
           print 'ready.'
        except java.lang.NullPointerException,e:
           self.label.setText( 'Please select a valid package (if there are no packages, create a new file):'+str(e))
           print 'Please select a valid package (if there are no packages, create a new file):',str(e)
        except java.lang.Exception,e:
           self.label.setText( 'Error while reverse engineering:'+str(e))
           print 'Error:',str(e)
           print 'Please ensure that you have installed the server-sided part of ZClassReverse. see the information at www.bluedynamics.com'
        except Exception,e:
           self.label.setText( 'Error while reverse engineering:'+str(e))
           print 'Error:',str(e)
           print 'Please ensure that you have installed the server-sided part of ZClassReverse. see the information at www.bluedynamics.com'
        except:
           self.label.setText('error')
           print 'error'

        self.dispose()
           
    def cancelIt(self,src):
        self.ok=0
        self.dispose()

def reverse(od):
   form=ReverseForm(od)
   form.show()
   

      